// B_PM23_SerialCmnd.ino
/*
There has to be a better way of telling Bambino what to do than keep
modifying this sketch.
Here we start by sending commands to bambino from the Serial Monitor,
or any terminal program.
The commands are 
'A'	- beAlive()   until reset      // the loop() from B_M22_BeAlive	
'F' - Forwards one step
'B' - Backwards one step
'R' - turn Right one step
'L' - turn Left one step
'H' - Halt with both feet together
'V' - Voice, make a beep
'S' - Stop/Start toggle reset flag stopped/walking
'U' - read Ultrasonic sensor and react to objectRange using avoidUObjects() behaviour rules
'W' - read Whisker sensors and avoid holes (if any) using avoidObstacles() behaviour rules
'G' - read Beard sensors and avoid obstacles (if any) using avoidHoles() behaviour rules
'?' - print instructions

The whiskerLEDs glowing dim indicate Bambino is ready for a command.
If other keys are pressed, including lower case keys, then the Whisker LEDs will flash.
*/

char thisrobot[] ="BambinoPM ";
char thisprog[]  ="B_PM23_SerialCmnd";
// Binary sketch size: 9,196 bytes (of a 30,720 byte maximum)

//== SECTIONS ===============================================
//== NEW 
//== CHANGED
//== INCLUDES
//     pinsPM20.h     - pin usage
//     varsConsPM23.h - EEPROM addresses, general, Ultrasonic module HC-SR04
//     EEPROM.h     - the Arduino EEPROM library
//== EEPROM
//== SETUP
//     initPM20.h     - Initialise various things
//== LOOP
//== FUNCTIONS
//   badkey()         // indicate invalid key
//   beAlive()        // the loop() from B_M22_BeAlive
//   USreact()        // read Ultrasonic sensor and react to objectRange using avoidUObjects() behaviour rules
//   WhiskerReact()   // read Whisker sensors and avoid holes (if any) using avoidObstacles() behaviour rules
//   groundReact()    // read Beard sensors and avoid obstacles (if any) using avoidHoles() behaviour rules
//   whiskerLEDsDim()
//TAB FILES
//   actsPM23.ino
//     ACTS - WanderSteps(steps), Wander()
//            RightStep(steps), LeftStep(steps)
//            rock(repeats), shuffle(repeats)
//     SOUND -  whee(), whoa()  
//   behaviourPM20.ino
//     BEHAVIOUR MODIFIERS - speeddown(), speedup()
//     BEHAVIOUR RULES - avoidUObjects(), avoidObstacles(), avoidHoles()
//   sensePM20.ino
//     SENSE - dist_HC_SR04()
//           - readWhiskers(), QueryObstacle(), showWhiskerLightLevels() 
//           - readBeard(), QueryEdge(), showBeardLightLevels() 
//   movement23.ino
//     META MOVEMENT - turnAway(), turnMemory(), turnOther() 
//     MOVEMENT - forward(), backward(), turnRight(), turnLeft(), 
//                halt(), 
//                rollCentre(), rollRight(), rollLeft(), 
//                paceCentre(), paceRightForward(), paceLeftForward(), paceRightBackward(), paceLeftBackward(), 
//                rollmove(), pacemove()
//   servos20.ino
//     SERVOS - ServosInitC()
//   hardware20.ino
//     HARDWARE - ServoPulse(byte Spin, int Spulse), beep(byte beeppin,int beepfreqHz,long beeptime)
//   system20.ino
//     SYSTEM - toggle_f_walking(), initialiseEEPROM()

//== NEW ====================================================
/*
void beAlive()         // the loop() from B_M22_BeAlive
void whiskerLEDsDim()  //indicate ready for command    
void USreact()    // read Ultrasonic sensor and react to objectRange ....
movement23.ino   - added halt()
actsPM23.ino     - added from B_M22_BeAlive.ino
                   RightStep(steps), LeftStep(steps),
                   rock(repeats), shuffle(repeats),
                   whee(), whoa()
varsConsPM23.h   - added byte cmnd =0;        // current command   
badkey()       // indicate invalid key
servos23.ino     - added enableServos() disableServos()
*/
//== CHANGED ================================================
/*
#include "varsConsPM20.h"  ->  #include "varsConsPM23.h"
movement20.ino -> movement23.ino
actsPM21.ino     -> actsPM23.ino
loop() - act on the single letter commands from the Serial Monitor or other terminal
*/
//== INCLUDES ===============================================
#include "pinsPM20.h"
#include "varsConsPM23.h"
#include <EEPROM.h>           // to read and write internal EEPROM
  // EEPROM.write(address,bytevalue)
  // bytevalue =EEPROM.read(address)
//== VARIABLES and CONSTANTS ================================           
//== SETUP ==================================================
void setup() 
{ 
  #include "initPM20.h" //Initialise various things
  
  //now configure Bambino
  f_forwards =1;    // set default
  rollspeed =1;     // start off slow
  pacespeed =1;     // start off slow
  rollspeedmax =4;  // quite fast 
  pacespeedmax =6;  // quite fast
}
//== LOOP ===================================================
void loop()
{ 
  whiskerLEDsDim();          //indicate ready for command    
  if (Serial.available()) 
  {
    cmnd =Serial.read();
    switch (cmnd) 
    {
      case 'F':  case 'f':  forward();      break;        
      case 'B':  case 'b':  backward();     break;        
      case 'R':  case 'r':  turnRight();    break;        
      case 'L':  case 'l':  turnLeft();     break;        
      case 'H':  case 'h':  halt();         break; 
      case 'V':  case 'v':  whee();         break; 
      case 'U':  case 'u':  USreact();      break; 
      case 'S':  case 's':  		
        toggle_f_walking();
        if (f_walking) {Serial.println(" walking");} else {Serial.println(" stopped");}
        break;    
      case 'W':  case 'w':  WhiskerReact(); break; 
      case 'G':  case 'g':  BeardReact();   break; 
      case 'A':	 case 'a':  beAlive();      break;     // do until reset
      case '?':	 printHelp();    break;     // print instructions
      default:   badkey();        		
    }    // end switch
  }  
}   
//== FUNCTIONS ==============================================
void badkey()     // indicate invalid key
{ digitalWrite(RwhiskerLEDpin_, LOW);// whisker LED ON
  delay(100);
  digitalWrite(RwhiskerLEDpin_, HIGH);// whisker LED OFF    
  digitalWrite(LwhiskerLEDpin_, LOW);// whisker LED ON
  delay(100);
  digitalWrite(LwhiskerLEDpin_, HIGH);// whisker LED OFF    
}  

void printHelp()
{ Serial.println("'F' - Forwards one step");
  Serial.println("'B' - Backwards one step");
  Serial.println("'R' - turn Right one step");
  Serial.println("'L' - turn Left one step");
  Serial.println("'H' - Halt with both feet together");
  Serial.println("'V' - Voice, make a beep");
  Serial.println("'S' - Stop/Start toggle reset flag stopped/walking");
  Serial.println("'U' - read Ultrasonic sensor and react");
  Serial.println("'W' - read Whisker sensors and avoid obstacles");
  Serial.println("'G' - read Beard sensors and avoid holes");
  Serial.println("'A' - beAlive()   until reset"); 
  Serial.println("'?' - print Help");     
}

void USreact()     // read Ultrasonic sensor and react to objectRange using avoidUObjects() behaviour rules
{ objectRange =dist_HC_SR04();
  Serial.print(" Ultrasonic distance =");
  Serial.println(objectRange);
  avoidUObjects();
}

void WhiskerReact()     // read Whisker sensors and avoid holes (if any) using avoidObstacles() behaviour rules
{ readWhiskers(); 
  queryObstacles();
  avoidObstacles();
}

void BeardReact()     // read Beard sensors and avoid obstacles (if any) using avoidHoles() behaviour rules
{ readBeard();
  queryEdge();
  avoidHoles(); 
}

void beAlive()         // the loop() from B_M22_BeAlive
{ while(true)
  { switch (random(9))
    { case 0: WanderSteps(random(10)); break;
      case 1: rock(random(3)); break;
      case 2: shuffle(random(3)); break;
      case 3: whoa(); break;
      case 4: whee(); break;
      case 5: rollspeed =random(1,4); break;
      case 6: pacespeed =random(1,6); break;
      case 7: RightStep(random(6)); break;
      case 8: LeftStep(random(6)); break;
    }
  }	
}

void whiskerLEDsDim()
{ analogWrite(RwhiskerLEDpin_,254);   //Right whiskerLED, LOW=on so 255 is off
  analogWrite(LwhiskerLEDpin_,254);   //Left whiskerLED, LOW=on so 255 is off
}  
//===========================================================










