// B_PM24_Learn.ino
/*

We can store in EEPROM a sequence of the commands used in B_PM23_SerialCmnd,
then replay it. These moves will be remembered even with power off.
 
The whiskerLEDs glowing dim indicate Bambino is ready for a command.
The redLED on indicate Bambino is recording commands.

The commands are 
'.',[Esc] - quit current operation - recording, playing
'S' - Stop/Start
'P' - Play moves Act, if included in a recording will restart the Act
'F' - Forwards one step
'B' - Backwards one step
'R' - turn Right one step
'L' - turn Left one step
'H' - Halt with both feet together
'V' - Voice, make a beep
'U' - read Ultrasonic sensor and react to objectRange using avoidUObjects() behaviour rules
'W' - read Whisker sensors and avoid holes (if any) using avoidObstacles() behaviour rules
'G' - read Beard sensors and avoid obstacles (if any) using avoidHoles() behaviour rules
'A' - beAlive()   until reset      // the loop() from B_M22_BeAlive
'r' - Record moves into EEPROM
'p' - Print moves in Act
'?' - print instructions

If other keys are pressed, including lower case keys, then the Whisker LEDs will flash badkey().
Upper-case is used for single letter commands and 
Lower-case will be used for parameter commands.

The Act slot has 32 bytes and can have up to 31 commands with the last entry '.' indicating end of Act.

Until recording mode is entered any of the commands have immediate effect.
In immediate mode keys 'P' will play Act, whatever has been recorded. 
If 'r' is entered then the Act will start to be recorded.
Bambino will 'bip' each time a command is entered in record mode.
Any of the keys can be entered in recording mode including 'r' but that 
makes no sense and is blocked during recording as is 'p' by setting f_recordC to false.
'S','?' operate but are also blocked from being recorded by setting f_recordC to false.
eg r,F,F,R,R,P,.    records FFRRP. in the Act 
if in immediate mode 'P' is entered then the Act will be played
giving Forward, Forward, turnRight, turnRight, Play
so the Act will repeat until '.' or Esc is keyed.
*/

char thisrobot[] ="BambinoPM ";
char thisprog[]  ="B_PM24_Learn";
// Binary sketch size: 10,086 bytes (of a 30,720 byte maximum)

//== SECTIONS ===============================================
//== NEW 
//== CHANGED
//== INCLUDES
//     pinsPM20.h     - pin usage
//     varsConsPM23.h - EEPROM addresses, general, Ultrasonic module HC-SR04
//     EEPROM.h       - the Arduino EEPROM library
//== EEPROM
//== SETUP
//     initPM20.h     - Initialise various things
//== LOOP
//== FUNCTIONS
//   Ccmnd <- PlayAct() 
//   PrintAllActs()
//   printAct()
//   setEEPROMindexes(char Act)      //  changes EEPROMindex, ActEnd
//   printHelp()
//   badkey()         // indicate invalid key
//   USreact()        // read Ultrasonic sensor and react to objectRange using avoidUObjects() behaviour rules
//   WhiskerReact()   // read Whisker sensors and avoid holes (if any) using avoidObstacles() behaviour rules
//   BeardReact()     // read Beard sensors and avoid obstacles (if any) using avoidHoles() behaviour rules
//   beAlive()        // the loop() from B_M22_BeAlive
//   redLEDon()
//   redLEDoff() 
//   bip()            // make a bip sound
//   whiskerLEDsDim()
//   whiskerLEDsOff()
//TAB FILES
//   actsPM23.ino
//     ACTS - WanderSteps(steps), Wander()
//            RightStep(steps), LeftStep(steps)
//            rock(repeats), shuffle(repeats)
//     SOUND -  whee(), whoa()  
//   behaviourPM20.ino
//     BEHAVIOUR MODIFIERS - speeddown(), speedup()
//     BEHAVIOUR RULES - avoidUObjects(), avoidObstacles(), avoidHoles()
//   sensePM20.ino
//     SENSE - dist_HC_SR04()
//           - readWhiskers(), QueryObstacle(), showWhiskerLightLevels() 
//           - readBeard(), QueryEdge(), showBeardLightLevels() 
//   movement23.ino
//     META MOVEMENT - turnAway(), turnMemory(), turnOther() 
//     MOVEMENT - forward(), backward(), turnRight(), turnLeft(), 
//                halt(), 
//                rollCentre(), rollRight(), rollLeft(), 
//                paceCentre(), paceRightForward(), paceLeftForward(), paceRightBackward(), paceLeftBackward(), 
//                rollmove(), pacemove()
//   servos20.ino
//     SERVOS - ServosInitC()
//   hardware20.ino
//     HARDWARE - ServoPulse(byte Spin, int Spulse), beep(byte beeppin,int beepfreqHz,long beeptime)
//   system20.ino
//     SYSTEM - toggle_f_walking(), initialiseEEPROM()

//== NEW ====================================================
/*
boolean f_recording =false;   //recording an Act
boolean f_recordC =false;     //record current Command
boolean f_play =false;        //Play Act
const byte Esc =27;           //Esc key value
int EEPROMindex;              // index to moves in an Act in second page of EEPROM
int ActEnd;                   // end address of current recording Act in EEPROM
int moves =0;                 // current move
char Ccmnd =0;                // current Character command, default null command 
char PlayAct() 
void printAct()
void setEEPROMindexes()
void bip()                    // make a bip sound
void redLEDon()
void redLEDoff()             
*/
//== CHANGED ================================================
/*
setup() -   Serial.print(">");         //ready prompt
loop()  - switchcase expanded
          section added at end to record Ccmnds - if (f_recording  && f_recordC)
          red LED on indicates recording 
          bip() when recording				  
void printHelp() - amended		  
*/

//== INCLUDES ===============================================
#include "pinsPM20.h"
#include "varsConsPM23.h"
#include <EEPROM.h>           // to read and write internal EEPROM
  // EEPROM.write(address,bytevalue)
  // bytevalue =EEPROM.read(address)
  
//== VARIABLES and CONSTANTS ================================
boolean f_recording =false;   //recording an Act
boolean f_recordC =false;     //record current Command
boolean f_play =false;        //Play Act
const byte Esc =27;           //Esc key value
int EEPROMindex;              // index to moves in an Act in second page of EEPROM
int ActEnd;                   // end address of current recording Act in EEPROM
int moves =0;                 // current move
char Ccmnd =0;                // current Character command, default null command
//== SETUP ==================================================
void setup() 
{ 
  #include "initPM20.h" //Initialise various things
  
  //now configure Bambino
  f_forwards =1;    // set default
  rollspeed =1;     // start off slow
  pacespeed =1;     // start off slow
  rollspeedmax =4;  // quite fast 
  pacespeedmax =6;  // quite fast
  
}
//== LOOP ===================================================
void loop()
{ Ccmnd =0;
  f_recordC =true;     // record Command, default =true
  whiskerLEDsDim();    //indicate ready for command  
  if(f_recording)  { redLEDon(); } else { redLEDoff(); }
  if(f_play)       { whiskerLEDsOff();  }
  if(!f_play)      { Serial.print(">"); }    //ready prompt
  
GetCcmnd:
  while(Ccmnd==0)
  { if(f_recording)  { while(!Serial.available()) {} Ccmnd =Serial.read();  bip();  }   //wait for Serial
    if(!f_recording) { if(Serial.available()) { Ccmnd =Serial.read(); }                 //over-ride if Serial  
                       else { if(f_play) {Ccmnd =PlayAct(); } } }                       //get Ccmnd 
  }					 

DoCommand:
  switch (Ccmnd) 
  { case Esc:  Ccmnd ='.';     //[Esc] key, fall through to '.'
    case '.':  if (f_recording) {  EEPROM.write(EEPROMindex,'.'); }     //terminated string
               f_recording =false;
               f_play =false;
               break;
    case 'A':  if (!f_recording) { beAlive(); }  break; // if recording record 'A' otherwise do beAlive() until reset
    case 'P':  if (!f_recording) { f_play =true;  setEEPROMindexes(); }  break;
    case 'F':  forward();      break;        
    case 'B':  backward();     break;        
    case 'R':  turnRight();    break;        
    case 'L':  turnLeft();     break;        
    case 'H':  halt();         break; 
    case 'V':  whee();         break;
    case 'S':  f_recordC =false;
               toggle_f_walking(); 
               if (f_walking) {Serial.println(" walking");} else {Serial.println(" stopped");}
               break;    	  
    case 'U':  USreact();      break; 
    case 'W':  WhiskerReact(); break; 
    case 'G':  BeardReact();   break;
    case 'r':  f_recordC =false;  
               if (!f_recording) 
	             { f_play =false;  f_recording =true;  setEEPROMindexes(); }
               break;
    case 'p':  f_recordC =false;  if (!f_recording) { printAct(); }  break;
    case '?':  f_recordC =false;  printHelp();  break;     // print instructions
    default:   f_recordC =false;  badkey();  Serial.println("bad key");  break;  //don't record bad key		
  }    // end switch
	
  if (f_recording  && f_recordC) 
  { EEPROM.write(EEPROMindex,Ccmnd);  EEPROMindex++;
    if (EEPROMindex>=ActEnd) { f_recording =false;  EEPROM.write(ActEnd,'.');  badkey(); }     // terminated string
  } 
}   
//== FUNCTIONS ==============================================
char PlayAct() 
{ char _Ccmnd =EEPROM.read(EEPROMindex++);
  if(_Ccmnd==255 || _Ccmnd==-1) _Ccmnd ='.';     //uninitialised EEPROM value, don't play it, change to '.'
  return _Ccmnd;
}  
//----------------------------
void printAct()
{ moves =0; 
  setEEPROMindexes();
  Serial.print("Act-"); 
  Ccmnd =EEPROM.read(EEPROMindex++);
  // -1 (=255) is uninitialised EEPROM value
  while(Ccmnd !='.' && Ccmnd !=-1) { Serial.print(Ccmnd);  moves++;  Ccmnd =EEPROM.read(EEPROMindex++); }
  Serial.print(". - ");
  Serial.print(moves);
  Serial.println(" moves");
  moves =0;
}
//----------------------------
void setEEPROMindexes()      // set EEPROM pointers
{ EEPROMindex =256;  ActEnd =EEPROMindex +31; }
//----------------------------
void badkey()     // indicate invalid key
{ digitalWrite(RwhiskerLEDpin_, LOW);// whisker LED ON
  delay(100);
  digitalWrite(RwhiskerLEDpin_, HIGH);// whisker LED OFF    
  digitalWrite(LwhiskerLEDpin_, LOW);// whisker LED ON
  delay(100);
  digitalWrite(LwhiskerLEDpin_, HIGH);// whisker LED OFF  
  whoa();  
}  
//----------------------------
void printHelp()
{
  Serial.println("'.' - stop playing or recording");
  Serial.println("'P' - Play or restart the Act");
  Serial.println("'F' - Forwards one step");
  Serial.println("'B' - Backwards one step");
  Serial.println("'R' - turn Right one step");
  Serial.println("'L' - turn Left one step");
  Serial.println("'H' - Halt with both feet together");
  Serial.println("'V' - Voice, make a beep");
  Serial.println("'S' - Stop/Start toggle reset flag stopped/walking");
  Serial.println("'U' - read Ultrasonic sensor and react");
  Serial.println("'W' - read Whisker sensors and avoid obstacles");
  Serial.println("'G' - read Beard sensors and avoid holes");
  Serial.println("'A' - beAlive()   until quit");
  Serial.println("'r' - Record moves into EEPROM");
  Serial.println("'p' - Print moves in EEPROM Act");
  Serial.println("'?' - these instructions");
}
//----------------------------
void USreact()     // read Ultrasonic sensor and react to objectRange using avoidUObjects() behaviour rules
{ objectRange =dist_HC_SR04();
  Serial.println();
  Serial.print("Ultrasonic distance =");
  Serial.println(objectRange);
  avoidUObjects();
}
//----------------------------
void WhiskerReact()     // read Whisker sensors and avoid holes (if any) using avoidObstacles() behaviour rules
{ readWhiskers(); 
  queryObstacles();
  avoidObstacles();
}
//----------------------------
void BeardReact()     // read Beard sensors and avoid obstacles (if any) using avoidHoles() behaviour rules
{ readBeard();
  queryEdge();
  avoidHoles(); 
}
//----------------------------
void beAlive()         // the loop() from B_M22_BeAlive
{ while(true)
  { switch (random(9))
    { case 0: WanderSteps(random(10)); break;
      case 1: rock(random(3)); break;
      case 2: shuffle(random(3)); break;
      case 3: whoa(); break;
      case 4: whee(); break;
      case 5: rollspeed =random(1,4); break;
      case 6: pacespeed =random(1,6); break;
      case 7: RightStep(random(6)); break;
      case 8: LeftStep(random(6)); break;
    } // end switch
	if(Serial.available()) { Ccmnd =Serial.read(); if(Ccmnd=='.' || Ccmnd==Esc) { Serial.print(">");  break; } }
  }	// end while
}
//----------------------------
void redLEDon()
{ digitalWrite(redLEDpin,HIGH); }  
//----------------------------
void redLEDoff()
{ digitalWrite(redLEDpin,LOW); }  
//----------------------------
void bip()            // make a bip sound
{ beep(voicePin,1500,50); }
//----------------------------    
void whiskerLEDsOff()
{ digitalWrite(RwhiskerLEDpin_, HIGH);                             // whisker LED OFF
  digitalWrite(LwhiskerLEDpin_, HIGH);                             // whisker LED OFF
}  
//----------------------------  
void whiskerLEDsDim()
{ analogWrite(RwhiskerLEDpin_,254);   //Right whiskerLED, LOW=on so 255 is off
  analogWrite(LwhiskerLEDpin_,254);   //Left whiskerLED, LOW=on so 255 is off
}
//===========================================================



















