// B_PM25_Learn8.ino
/*

We can store in EEPROM sequences of the commands used in B_PM23_SerialCmnd,
then replay them.
If each sequence which we will call an Act takes less than 32 commands 
then 8 routines will fit in the second 256 byte page of the EEPROM.

The whiskerLEDs glowing dim indicate Bambino is ready for a command.
The redLED on indicate Bambino is recording commands.

The commands are 
'.',[Esc] - quit current operation - recording, playing, input
'S' - Stop/Start
'P' - Print moves in all Acts
'1'-'8' the routine or Act to record or replay
'F' - Forwards one step
'B' - Backwards one step
'R' - turn Right one step
'L' - turn Left one step
'H' - Halt with both feet together
'V' - Voice, make a beep
'U' - read Ultrasonic sensor and react to objectRange using avoidUObjects() behaviour rules
'W' - read Whisker sensors and avoid holes (if any) using avoidObstacles() behaviour rules
'G' - read Beard sensors and avoid obstacles (if any) using avoidHoles() behaviour rules
'A' - beAlive()   until reset      // the loop() from B_M22_BeAlive
'r','1'-'8' - Record moves into selected Act in EEPROM
'p','1'-'8' - Print moves in selected Act '1' - '8'
'?' - print instructions

If other keys are pressed, including lower case keys, then the Whisker LEDs will flash badkey().
Upper-case is used for single letter commands and 
Lower-case is used for parameter commands.

Each Act slot has 32 bytes and can have up to 31 commands with the last entry set to 0 indicating end of Act.

Until programming mode is entered any of the commands have immediate effect.
In immediate mode keys 1 - 8 will play the corresponding Act, whatever has been recorded. 
If 'r' is entered then the next key must be 1 - 8 to select the Act to be recorded.
If '.' is entered instead then recording will not start.
Any of the keys can be entered in recording mode including 'r' but that 
makes no sense and is blocked during recording as is 'p' by setting f_recordC to false.
'S','?' operate but are also blocked from being recorded by setting f_recordC to false.
When recording has started keys 1 - 8 will record a jump to the corresponding Act.
However in record mode playback of that Act is suppressed, hence after entering an Act 
number it makes no sense to add more commands since they will never get done
because control will jump to the new Act; so just enter '.', or '.' to quit recording.
eg r,1,F,F,R,R,1,Q  or r,1,F,F,R,R,1,.  records FFRR1. in Act1 
if in immediate mode '1' is entered then Act1 will be played giving
Forward, Forward, turnRight, turnRight, jump to Act 1
So Act 1 will play forever, Act 1 could also be invoked from another Act
eg r,2,1,Q       here Act 2 does nothing except jump to Act 1.
'.' or Esc quits a playing Act.
*/

char thisrobot[] ="BambinoPM ";
char thisprog[]  ="B_PM25_Learn8";
// Binary sketch size: 10,126 bytes (of a 30,720 byte maximum)

//== SECTIONS ===============================================
//== NEW 
//== CHANGED
//== INCLUDES
//     pinsPM20.h     - pin usage
//     varsConsPM23.h - EEPROM addresses, general, Ultrasonic module HC-SR04
//     EEPROM.h     - the Arduino EEPROM library
//== EEPROM
//== SETUP
//     initPM20.h     - Initialise various things
//== LOOP
//== FUNCTIONS
//   Ccmnd <- PlayAct() 
//   PrintAllActs()
//   printAct()
//   Act <- getAct() 
//   setEEPROMindexes(char Act)      //  changes EEPROMindex, ActEnd
//   printHelp()
//   badkey()         // indicate invalid key
//   USreact()        // read Ultrasonic sensor and react to objectRange using avoidUObjects() behaviour rules
//   WhiskerReact()   // read Whisker sensors and avoid holes (if any) using avoidObstacles() behaviour rules
//   BeardReact()     // read Beard sensors and avoid obstacles (if any) using avoidHoles() behaviour rules
//   beAlive()        // the loop() from B_M22_BeAlive
//   redLEDon()
//   redLEDoff() 
//   bip()            // make a bip sound
//   whiskerLEDsDim()
//   whiskerLEDsOff()
//TAB FILES
//   actsPM23.ino
//     ACTS - WanderSteps(steps), Wander()
//            RightStep(steps), LeftStep(steps)
//            rock(repeats), shuffle(repeats)
//     SOUND -  whee(), whoa()  
//   behaviourPM20.ino
//     BEHAVIOUR MODIFIERS - speeddown(), speedup()
//     BEHAVIOUR RULES - avoidUObjects(), avoidObstacles(), avoidHoles()
//   sensePM25.ino
//     SENSE - dist_HC_SR04()
//           - readWhiskers(), QueryObstacle(), showWhiskerLightLevels() 
//           - readBeard(), QueryEdge(), showBeardLightLevels() 
//   movement23.ino
//     META MOVEMENT - turnAway(), turnMemory(), turnOther() 
//     MOVEMENT - forward(), backward(), turnRight(), turnLeft(), 
//                halt(), 
//                rollCentre(), rollRight(), rollLeft(), 
//                paceCentre(), paceRightForward(), paceLeftForward(), paceRightBackward(), paceLeftBackward(), 
//                rollmove(), pacemove()
//   servos20.ino
//     SERVOS - ServosInitC()
//   hardware20.ino
//     HARDWARE - ServoPulse(byte Spin, int Spulse), beep(byte beeppin,int beepfreqHz,long beeptime)
//   system20.ino
//     SYSTEM - toggle_f_walking(), initialiseEEPROM()

//== NEW ====================================================
/*
char Act;                     //current Act as '1' - '8'
byte actIndex =0;             // current Act number, set to default 
void PrintAllActs()        
void getAct()   
*/
//== CHANGED ================================================
/*
loop() - case 'P' changed from Play to Print all Acts
         case '1' - '8' added to select more acts and to play them
         case 'r' now needs to be followed by an act number '1'-'8'		 
         case 'p' now needs to be followed by an act number '1'-'8'	
void printAct() - changed to print Act number
void setEEPROMindexes()   is now   void setEEPROMindexes(char Act)
void printHelp() - amended instructions   		
sensePM20.ino changed to sensePM25.ino 
  - dist_HC_SR04() removed if (debug) {Serial.print('U');}
  - showWhiskerLightLevels() - added  Serial.println();
  - showBeardLightLevels()   - added  Serial.println();
*/

//== INCLUDES ===============================================
#include "pinsPM20.h"
#include "varsConsPM23.h"
#include <EEPROM.h>           // to read and write internal EEPROM
  // EEPROM.write(address,bytevalue)
  // bytevalue =EEPROM.read(address)
  
//== VARIABLES and CONSTANTS ================================
boolean f_recording =false;   //recording an Act
boolean f_recordC =false;     //record current Command
boolean f_play =false;        //Play current Act
const byte Esc =27;           //Esc key value
char Act;                     //current Act as '1' - '8'
byte actIndex =0;             // current Act number, set to default           
int EEPROMindex;              // index to moves in an Act in second page of EEPROM
int ActEnd;                   // end address of current recording Act in EEPROM
int moves =0;                 // current move
char Ccmnd =0;                // current Character command, default null command
//== SETUP ==================================================
void setup() 
{ 
  #include "initPM20.h" //Initialise various things
  
  //now configure Bambino
  f_forwards =1;    // set default
  rollspeed =1;     // start off slow
  pacespeed =1;     // start off slow
  rollspeedmax =4;  // quite fast 
  pacespeedmax =6;  // quite fast
  
}
//== LOOP ===================================================
void loop()
{ Ccmnd =0;
  f_recordC =true;     // record Command, default =true
  whiskerLEDsDim();    //indicate ready for command  
  if(f_recording)  { redLEDon(); } else { redLEDoff(); }
  if(f_play)       { whiskerLEDsOff();  }
  if(!f_play)      { Serial.print(">"); }    //ready prompt
  
GetCcmnd:
  while(Ccmnd==0)
  { if(f_recording)  { while(!Serial.available()) {} Ccmnd =Serial.read();  bip();  }   //wait for Serial
    if(!f_recording) { if(Serial.available()) { Ccmnd =Serial.read(); }                 //over-ride if Serial  
                       else { if(f_play) {Ccmnd =PlayAct(); } } }                       //get Ccmnd 
  }						 

DoCommand:
  Serial.print(Ccmnd); 
  switch (Ccmnd) 
  { case Esc:  Ccmnd ='.';     //[Esc] key, fall through to '.'
    case '.':  if (f_recording) {  EEPROM.write(EEPROMindex,'.'); }     //terminated string
               f_recording =false;
               f_play =false;
               break;
    case 'P':  if (!f_recording) { PrintAllActs(); }  break;
    case 'A':	 if (!f_recording) { beAlive(); }  break; // if recording record 'A' otherwise do beAlive() until reset
    case '1':  // fall through
    case '2':  // fall through
    case '3':  // fall through
    case '4':  // fall through
    case '5':  // fall through
    case '6':  // fall through
    case '7':  // fall through
    case '8':  if (!f_recording) { f_play =true; Act =Ccmnd;  setEEPROMindexes(Act); }  break;
    case 'F':  forward();      break;        
    case 'B':  backward();     break;        
    case 'R':  turnRight();    break;        
    case 'L':  turnLeft();     break;        
    case 'H':  halt();         break; 
    case 'V':  whee();         break;
    case 'S':  f_recordC =false;
               toggle_f_walking(); 
               if (f_walking) {Serial.println(" walking");} else {Serial.println(" stopped");}
               break;    	
    case 'U':  USreact();      break; 
    case 'W':  WhiskerReact(); break; 
    case 'G':  BeardReact();   break;
    case 'r':  f_recordC =false;
	           redLEDon();
	           if (!f_recording) 
	             { getAct();  if(Ccmnd=='.') { break; }  
                   setEEPROMindexes(Act);  f_recording =true; 
				 }
               break;
    case 'p':  f_recordC =false;  
               if (!f_recording) 
                 { getAct();  if(Ccmnd=='.') { break; }  
                   printAct();  break;
				 }
    case '?':  f_recordC =false;  printHelp();  break;     // print instructions
    default:   f_recordC =false;  badkey();  Serial.println("bad key");  break;  //don't record bad key		
  }    // end switch
	
  if (f_recording  && f_recordC) 
  { EEPROM.write(EEPROMindex,Ccmnd);  EEPROMindex++;
    if (EEPROMindex>=ActEnd) { f_recording =false;  EEPROM.write(ActEnd,'.');  badkey(); }     // terminated string
  } 
}   
//== FUNCTIONS ==============================================
char PlayAct() 
{ char _Ccmnd =EEPROM.read(EEPROMindex++);
  if(_Ccmnd==255 || _Ccmnd==-1) _Ccmnd ='.';     //uninitialised EEPROM value, don't play it, change to '.'
  return _Ccmnd;
}  
//----------------------------
void PrintAllActs()
{ for(Act ='1'; Act<='8'; Act++) { printAct(); }  Serial.println(); }
//----------------------------
void printAct()
{ setEEPROMindexes(Act);
  Serial.println();    
  Serial.print("Act");
  Serial.print(Act);
  Serial.print(" ");
  Ccmnd =EEPROM.read(EEPROMindex++);
  // -1 (=255) is uninitialised EEPROM value
  while(Ccmnd !='.' && Ccmnd !=-1) { Serial.print(Ccmnd); moves++; Ccmnd =EEPROM.read(EEPROMindex++); }
  EEPROMindex -=1;        //step back to end of record just in case playing
  Serial.print(". - ");
  Serial.print(moves);
  Serial.print(" moves");
  moves =0;	
}
//----------------------------  
void getAct()
{ Ccmnd =0;
  do
  { if(Serial.available()) 
    { Ccmnd =Serial.read();
      switch (Ccmnd) 
      { case Esc:   Ccmnd ='.';     //[Esc] key, fall through to '.'
        case '.':  Serial.print(">");  break;     // quit, print ready prompt
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':  Act =Ccmnd;  break;
        default: badkey(); Serial.print(" enter Act or '.'"); Ccmnd =0;	
      }  //end case
    } //end if Serial
  } while(Ccmnd==0);
  Serial.print(Ccmnd);
}    
//----------------------------
void setEEPROMindexes(char Act)     // set EEPROM pointers
{ actIndex =Act -'1';               // ASCII to numeric,  Act 1 == actIndex 0
  EEPROMindex =256 +(actIndex*32);
  ActEnd =EEPROMindex +31;
}  
//----------------------------
void badkey()     // indicate invalid key
{ digitalWrite(RwhiskerLEDpin_, LOW);// whisker LED ON
  delay(100);
  digitalWrite(RwhiskerLEDpin_, HIGH);// whisker LED OFF    
  digitalWrite(LwhiskerLEDpin_, LOW);// whisker LED ON
  delay(100);
  digitalWrite(LwhiskerLEDpin_, HIGH);// whisker LED OFF  
  whoa();  
}  
//----------------------------
void printHelp()
{
  Serial.println("'1'-'8' the routine or Act to record or replay");
  Serial.println("'.',[Esc] - quit playing or recording");
  Serial.println("'P' - Print moves in all Acts");
  Serial.println("'F' - Forwards one step");
  Serial.println("'B' - Backwards one step");
  Serial.println("'R' - turn Right one step");
  Serial.println("'L' - turn Left one step");
  Serial.println("'H' - Halt with both feet together");
  Serial.println("'V' - Voice, make a beep");
  Serial.println("'S' - Stop/Start toggle reset flag stopped/walking");
  Serial.println("'U' - read Ultrasonic sensor and react");
  Serial.println("'W' - read Whisker sensors and avoid obstacles");
  Serial.println("'G' - read Beard sensors and avoid holes");
  Serial.println("'A' - beAlive()   until quit");
  Serial.println("'r','1'-'8' - Record moves into EEPROM");
  Serial.println("'p','1'-'8' - Print moves in selected Act '1' - '8'");
  Serial.println("'?' - these instructions");
}
//----------------------------
void USreact()     // read Ultrasonic sensor and react to objectRange using avoidUObjects() behaviour rules
{ objectRange =dist_HC_SR04();
  Serial.println();
  Serial.print("Ultrasonic distance =");
  Serial.println(objectRange);
  avoidUObjects();
}
//----------------------------
void WhiskerReact()     // read Whisker sensors and avoid obstacles (if any) using avoidObstacles() behaviour rules
{ readWhiskers(); 
  queryObstacles();
  avoidObstacles();
}
//----------------------------
void BeardReact()     // read Beard sensors and avoid holes (if any) using avoidHoles() behaviour rules
{ readBeard();
  queryEdge();
  avoidHoles(); 
}
//----------------------------
void beAlive()         // the loop() from B_PM22_BeAlive
{ while(true)
  { switch (random(9))
    { case 0: WanderSteps(random(10)); break;
      case 1: rock(random(3)); break;
      case 2: shuffle(random(3)); break;
      case 3: whoa(); break;
      case 4: whee(); break;
      case 5: rollspeed =random(1,4); break;
      case 6: pacespeed =random(1,6); break;
      case 7: RightStep(random(6)); break;
      case 8: LeftStep(random(6)); break;
    } // end switch
	if(Serial.available()) { Ccmnd =Serial.read(); if(Ccmnd=='.' || Ccmnd==Esc) { Serial.print(">");  break; } }
  }	// end while
}  
//----------------------------
void redLEDon()
{ digitalWrite(redLEDpin,HIGH); }  
//----------------------------
void redLEDoff()
{ digitalWrite(redLEDpin,LOW); }  
//----------------------------
void bip()            // make a bip sound
{ beep(voicePin,1500,50); }
//----------------------------    
void whiskerLEDsOff()
{ digitalWrite(RwhiskerLEDpin_, HIGH);                             // whisker LED OFF
  digitalWrite(LwhiskerLEDpin_, HIGH);                             // whisker LED OFF
}  
//----------------------------  
void whiskerLEDsDim()
{ analogWrite(RwhiskerLEDpin_,254);   //Right whiskerLED, LOW=on so 255 is off
  analogWrite(LwhiskerLEDpin_,254);   //Left whiskerLED, LOW=on so 255 is off
}
//===========================================================


















